#include <hidef.h>      /* common defines and macros */
#include <mc9s12a64.h>     /* derivative information */

#include "pll.h"
#include "timer.h"
#include "sci.h"
#include "gps.h"
#include "sd.h"

#pragma LINK_INFO DERIVATIVE "mc9s12a64"

// debug testing for the GPS module
void TestGPS(void);

// output the binary offsets of the GPS data type
void PrintGPSOffset(void);

// simple debug testing of the MicroSD card
void TestSD(void);

// Store GPS track to the SD card
void TestTracking(void);


// sets port pins that aren't padded out to have pull resistors
void UnusedInit(void);



unsigned char SD_Buffer[SD_BLOCK_SIZE];

void PrintTime(unsigned long t_s, unsigned long t_e) 
{
   t_e -= t_s;
   SCI_OutString(1, "Time: ");
   SCI_OutULInt(1, t_e);
   SCI_OutString(1, "\r\n");
}


// offset to start GPS track outputting
#define BLOCK_GPS_TRACK_OUT 1024




void main(void) { 
   UnusedInit();
   PLL_Init();
   Timer_Init(0,1);
   SCI_Init(1, 115200); // LCD / PC connection   
   GPS_Init();
   SD_Init();
   
   EnableInterrupts;
   
   DDRT |= 0xFF;     // PortT bits 0-7 are output
   
   PTT = 0;
   
   //PrintGPSOffset();
   //TestGPS();
   //TestSD();
   TestTracking();
   
   
   // spin forever
   for (;;);
}

// sets port pins that aren't padded out to have pull resistors
void UnusedInit(void)
{
   PERH = 0xFF;
   PERP = 0b01000000;
   PERS = 0xF0;
   
}



// Store GPS track to the SD card
void TestTracking(void) 
{	    
   unsigned short i;
   unsigned long count = 0;   
   int max, bufcount=0;	 
    
   	  
   i = SD_Card_Init();
   
   // determine how many points will fit in a block
   // max = the max index
   max = (SD_BLOCK_SIZE / sizeof(GPS_DATA)) - 1;
   
   
     
   for (i=0; i<SD_BLOCK_SIZE; i++) SD_Buffer[i] = 0;
   // clear the header
   // i = SD_Write_Block(SD_Buffer, BLOCK_GPS_TRACK_OUT);
   
   // read the header
   i = SD_Read_Block(SD_Buffer, BLOCK_GPS_TRACK_OUT);
   
   // copy out the first 4 bytes as an integer
   count = *(unsigned long *)SD_Buffer;
   
   for (;;)
   {	
      // new gps data ready
      if (GPS_ReadyFlag()) 
      {
         // translate the NMEA sentences to binary
         if (GPS_Translate()) 
         {
            // only record it if all of the position and bearing fields refreshed, and 3D   
            
            // copy the waypoint data into the SD buffer
            GPS_Value(&((GPS_DATA *)SD_Buffer)[bufcount]);
            bufcount++;
            
            if (bufcount >= max)
            {
               SCI_OutString(1, "Write "); 
               SCI_OutULInt(1, count);
               SCI_OutString(1, "\r\n");
            
               bufcount = 0;
               
               // write the data to the SD card
               count++;
               i = SD_Write_Block(SD_Buffer, BLOCK_GPS_TRACK_OUT + count);
               
               // write the count back to the header
               *(unsigned long *)SD_Buffer = count;
               i = SD_Write_Block(SD_Buffer, BLOCK_GPS_TRACK_OUT);
            } 
         }
         
      }     
   }
   
}






// debug testing for the GPS module
void TestGPS(void) 
{
   for(;;) 
   {
      PTT ^= 0x01;
      
      // wait for a byte from the GPS module
      // while (!SCI0_DATA_READY);
      
      // call handler as if triggered by an interrupt
      // GPS_Receive_Handler();
      
      if (GPS_ReadyFlag()) 
      {
         GPS_Translate();
      }                       
   }   
}

// output the binary offsets of the GPS data type
// used to help translate from the codewarrior compiler to gcc storage mechanism
void PrintGPSOffset(void)
{
   GPS_DATA g;

   SCI_OutString(1, "sizeof(GPS_DATA) = "); SCI_OutUInt(1, sizeof(GPS_DATA)); SCI_OutString(1, "\r\nOffsets:\r\n");
   SCI_OutString(1, "latitude: ");  SCI_OutULInt(1, (unsigned long)&(g.latitude) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "longitude: ");  SCI_OutULInt(1, (unsigned long)&(g.longitude) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "altitude: ");  SCI_OutULInt(1, (unsigned long)&(g.altitude) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "speed: ");  SCI_OutULInt(1, (unsigned long)&(g.speed) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "bearing: ");  SCI_OutULInt(1, (unsigned long)&(g.bearing) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "numSV: ");  SCI_OutULInt(1, (unsigned long)&(g.numSV) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "year: ");  SCI_OutULInt(1, (unsigned long)&(g.year) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "month: ");  SCI_OutULInt(1, (unsigned long)&(g.day) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "day: ");  SCI_OutULInt(1, (unsigned long)&(g.day) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "hr: ");  SCI_OutULInt(1, (unsigned long)&(g.hr) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "min: ");  SCI_OutULInt(1, (unsigned long)&(g.min) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "sec: ");  SCI_OutULInt(1, (unsigned long)&(g.sec) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "geoidSep: ");  SCI_OutULInt(1, (unsigned long)&(g.geoidSep) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "PDOP: ");  SCI_OutULInt(1, (unsigned long)&(g.PDOP) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "HDOP: ");  SCI_OutULInt(1, (unsigned long)&(g.HDOP) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "VDOP: ");  SCI_OutULInt(1, (unsigned long)&(g.VDOP) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "fix: ");  SCI_OutULInt(1, (unsigned long)&(g.fix) - (unsigned long)&g); SCI_OutString(1, "\r\n");
   SCI_OutString(1, "navMode: ");  SCI_OutULInt(1, (unsigned long)&(g.navMode) - (unsigned long)&g); SCI_OutString(1, "\r\n");
}


// simple debug testing of the MicroSD card
void TestSD(void) 
{	    
   unsigned short i,j;	
  
   char *replace = "OMG, h4x0red!!!";
   char *pos;
   
   // for timing purposes
   unsigned long t[4];
   
     
   SCI_OutString(1, "Initializing SD card...\r\n");
   t[0] = TimeStamp();
   i = SD_Card_Init();
   t[1] = TimeStamp();
   PrintTime(t[0], t[1]);
   
  
   
   SCI_OutString(1, "Reading block 645\r\n");
   t[0] = TimeStamp();
   i = SD_Read_Block(SD_Buffer, 645);   
   t[1] = TimeStamp();
   PrintTime(t[0], t[1]);
   
   SCI_OutString(1, "Before: ");
   for (i=0; i<30; i++)
   {
      SCI_OutChar(1, SD_Buffer[i]);
   }
   SCI_OutString(1, "\r\n");
   
   // modify the buffer
   pos = replace;
   i = 6;
   while (*replace) 
   {
      SD_Buffer[i] = *replace;
      replace++;
      i++;
   }  
   
   SCI_OutString(1, "After:  ");
   for (i=0; i<30; i++)
   {
      SCI_OutChar(1, SD_Buffer[i]);
   }
   SCI_OutString(1, "\r\n");
   
   //for (;;);
      
   
   SCI_OutString(1, "Writing edited block 645\r\n");  
   t[0] = TimeStamp();  
   i = SD_Write_Block(SD_Buffer, 645);
   t[1] = TimeStamp();
   PrintTime(t[0], t[1]);
   
   
   for (j=645; j<1000; j++)
   {	
      t[0] = TimeStamp();
      i = SD_Read_Block(SD_Buffer, j);
      t[1] = TimeStamp();
      PrintTime(t[0], t[1]);
           		      
      SCI_OutString(1, "Contents of memory block ");
      SCI_OutUInt(1, j);
      SCI_OutString(1, "\r\n");
      
      // print out a hex dump
      for (i=1; i<=SD_BLOCK_SIZE; i++)
      {
         SCI_OutHex(1, SD_Buffer[i-1]);
         if ( (i%16) == 0 )
            SCI_OutString(1, "\r\n");
         else if ( (i%8) == 0 )
            SCI_OutString(1, "  ");
         else
            SCI_OutChar(1, ' ');
      }
      SCI_OutString(1, "\r\n");
      SCI_OutString(1, "\r\n");
      
      // wait for key press
      i = SCI_InChar(1);
            
   }
}
