/******************************* timer.c *******************************/

// Sets up Timer subsystem and provides some timing functions

#include <hidef.h>
#include <mc9s12a64.h>
#include "constants.h"
#include "timer.h" 




// used as a high clock counter, number of overflows
unsigned short TOFCount;

PARTLONG temptime;


// Initializes timer subsystem
// to be used for standard timer, or TOF functionality
void Timer_Init(unsigned char PrescalerDivisor, int UseTOF)
{
   TSCR1 = 0x80;  // enable timer
   TSCR2 = PrescalerDivisor;  // timer prescaler frequency set to system clock freq / 2^n   
   PACTL = 0;     // turn off pulse accumulation mode
   
   if (UseTOF)
   {	        
      TFLG2 = 0x80;  // clear timer overflow flag
      TSCR2 |= 0x80;  // enable timer overflow interrupt
      //TOFCount = 0;
      EnableInterrupts
   }
} 	 													  


// interrupt for Timer Overflow
interrupt 16 void TOF_Handler(void)  
{	    
   TFLG2 = 0x80;  // clear timer overflow flag
   TOFCount++;   // increment TOF count   
}																				



// Blocking wait for a given number of clock cycles
// not completely accurate due to function call overhead
void Timer_Wait(unsigned short delay)
{ 
   unsigned short startTime;
   
   startTime = TCNT;
   while((TCNT-startTime) < delay); 
}


// Wait for given number of milliseconds
void Timer_Wait1ms(unsigned short delay)
{ 
   for(;delay>0;delay--)
   {
      Timer_Wait(CLOCK_SPEED/1000);
   }
}


// returns the current number of clock ticks in a 32bit int
unsigned long int TimeStamp(void)
{	  
   
   temptime.Half.Low = TCNT;
   temptime.Half.High = TOFCount;
   return temptime.Full;
}


// return TOFCount
unsigned short TOFNow(void)
{
   return TOFCount;
}
