/******************************* servo.h *******************************/

#ifndef HEADER_SERVO_H
#define HEADER_SERVO_H


// range of values accepted as input to servo control functions
#define SERVO_IN_MAX (1500)
#define SERVO_IN_MIN (-1500)

// number of prescaled clock cycles in one period
// used to achieve a 50Hz PWM for the servos
// value is dependent upon the system bus clock frequency
#define SERVO_PWM_PERIOD 60000

// allowable range of servo duty cycles
// correspond to approximately 1ms to 2ms, with tweaking 
// for individual servo differences
#define SERVO_OUT_PWM_MAX 6000
#define SERVO_OUT_PWM_ZERO 4500
#define SERVO_OUT_PWM_MIN 3000



// sets up the PWM registers for 20ms period and servo centered initial duty
void Servo_Init(void);

// sets up the PWM registers for 8bit operation on PTT0,1
void Servo_Init8(void);

// sets the duty cycle of selected 8bit PWM channel
void PWM_Duty8(unsigned char channel, unsigned char value);

// sets the duty cycle of selected 16bit PWM channel
// only two 16bit channels are available
void PWM_Duty16(unsigned char channel, unsigned short value);

// sets the period of selected 16bit PWM channel
void PWM_Period16(unsigned char channel, unsigned short value);

// sets the pulse width of a servo channel
// this corresponds to a 16bit PWM channel
// input values are SERVO_IN_MIN to SERVO_IN_MAX
void Servo_Duty(unsigned char channel, short value);

#endif
